@@ -5,18 +5,13 @@
 
 package org.jboss.as.connector.services.workmanager;
 
-import java.lang.invoke.MethodHandles;
+import java.util.concurrent.Executor;
 
 import org.jboss.as.threads.ManagedJBossThreadPoolExecutorService;
 import org.jboss.as.threads.ManagedQueueExecutorService;
 import org.jboss.as.threads.ManagedQueuelessExecutorService;
 import org.jboss.as.threads.ManagedScheduledExecutorService;
-import org.jboss.jca.core.CoreLogger;
 import org.jboss.jca.core.api.workmanager.StatisticsExecutor;
-import org.jboss.logging.Logger;
-import org.jboss.threads.BlockingExecutor;
-import org.jboss.threads.JBossThreadPoolExecutor;
-import org.jboss.threads.management.ThreadPoolExecutorMBean;
 
 /**
  * A StatisticsExecutor implementation keeping track of numberOfFreeThreads
@@ -25,21 +20,15 @@ import org.jboss.threads.management.ThreadPoolExecutorMBean;
  */
 
 public class StatisticsExecutorImpl implements StatisticsExecutor {
-    /**
-     * The logger
-     */
-    private static CoreLogger log = Logger.getMessageLogger(
-            MethodHandles.lookup(), CoreLogger.class,
-            org.jboss.jca.core.workmanager.StatisticsExecutorImpl.class.getName());
 
-    private final BlockingExecutor realExecutor;
+    private final Executor realExecutor;
 
     /**
      * StatisticsExecutorImpl constructor
      *
      * @param realExecutor the real executor we are delegating
      */
-    public StatisticsExecutorImpl(BlockingExecutor realExecutor) {
+    public StatisticsExecutorImpl(Executor realExecutor) {
         this.realExecutor = realExecutor;
     }
 
@@ -51,14 +40,6 @@ public class StatisticsExecutorImpl implements StatisticsExecutor {
 
     @Override
     public long getNumberOfFreeThreads() {
-        if (realExecutor instanceof JBossThreadPoolExecutor) {
-            return (long) ((JBossThreadPoolExecutor) realExecutor).getMaximumPoolSize()
-                    - ((JBossThreadPoolExecutor) realExecutor).getActiveCount();
-        }
-        if (realExecutor instanceof ThreadPoolExecutorMBean) {
-            return (long) ((ThreadPoolExecutorMBean) realExecutor).getMaxThreads()
-                    - ((ThreadPoolExecutorMBean) realExecutor).getCurrentThreadCount();
-        }
         if (realExecutor instanceof ManagedQueueExecutorService) {
             return (long) ((ManagedQueueExecutorService) realExecutor).getMaxThreads()
                     - ((ManagedQueueExecutorService) realExecutor).getCurrentThreadCount();
